      SUBROUTINE PCTZRZRV( M, N, A, IA, JA, DESCA, TAU, WORK )
*
*  -- ScaLAPACK routine (version 1.2) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     May 10, 1996
*
*     .. Scalar Arguments ..
      INTEGER            IA, JA, M, N
*     ..
*     .. Array Arguments ..
      INTEGER            DESCA( * )
      COMPLEX            A( * ),  TAU( * ), WORK( * )
*     ..
*
*  Purpose
*  =======
*
*  PCTZRZRV computes sub( A ) = A(IA:IA+M-1,JA:JA+N-1) from T, Z
*  computed by PCTZRZF.
*
*  Notes
*  =====
*
*  A description vector is associated with each 2D block-cyclicly dis-
*  tributed matrix.  This vector stores the information required to
*  establish the mapping between a matrix entry and its corresponding
*  process and memory location.
*
*  In the following comments, the character _ should be read as
*  "of the distributed matrix".  Let A be a generic term for any 2D
*  block cyclicly distributed matrix.  Its description vector is DESCA:
*
*  NOTATION        STORED IN      EXPLANATION
*  --------------- -------------- --------------------------------------
*  DT_A   (global) DESCA( DT_ )   The descriptor type.
*  CTXT_A (global) DESCA( CTXT_ ) The BLACS context handle, indicating
*                                 the BLACS process grid A is distribu-
*                                 ted over. The context itself is glo-
*                                 bal, but the handle (the integer
*                                 value) may vary.
*  M_A    (global) DESCA( M_ )    The number of rows in the distributed
*                                 matrix A.
*  N_A    (global) DESCA( N_ )    The number of columns in the distri-
*                                 buted matrix A.
*  MB_A   (global) DESCA( MB_ )   The blocking factor used to distribute
*                                 the rows of A.
*  NB_A   (global) DESCA( NB_ )   The blocking factor used to distribute
*                                 the columns of A.
*  RSRC_A (global) DESCA( RSRC_ ) The process row over which the first
*                                 row of the matrix A is distributed.
*  CSRC_A (global) DESCA( CSRC_ ) The process column over which the
*                                 first column of A is distributed.
*  LLD_A  (local)  DESCA( LLD_ )  The leading dimension of the local
*                                 array storing the local blocks of the
*                                 distributed matrix A.
*                                 LLD_A >= MAX(1,LOCp(M_A)).
*
*  Let K be the number of rows or columns of a distributed matrix,
*  and assume that its process grid has dimension p x q.
*  LOCp( K ) denotes the number of elements of K that a process
*  would receive if K were distributed over the p processes of its
*  process column.
*  Similarly, LOCq( K ) denotes the number of elements of K that a
*  process would receive if K were distributed over the q processes of
*  its process row.
*  The values of LOCp() and LOCq() may be determined via a call to the
*  ScaLAPACK tool function, NUMROC:
*          LOCp( M ) = NUMROC( M, MB_A, MYROW, RSRC_A, NPROW ),
*          LOCq( N ) = NUMROC( N, NB_A, MYCOL, CSRC_A, NPCOL ).
*
*  Arguments
*  =========
*
*  M       (global input) INTEGER
*          The number of rows to be operated on, i.e. the number of rows
*          of the distributed submatrix sub( A ). M >= 0.
*
*  N       (global input) INTEGER
*          The number of columns to be operated on, i.e. the number of
*          columns of the distributed submatrix sub( A ). N >= M >= 0.
*
*  A       (local input/local output) COMPLEX pointer into the
*          local memory to an array of dimension (LLD_A, LOCq(JA+N-1)).
*          On entry, sub( A ) contains the the factors T and Z computed
*          by PCTZRZF. On exit, the original matrix is restored.
*
*  IA      (global input) INTEGER
*          A's global row index, which points to the beginning of the
*          submatrix which is to be operated on.
*
*  JA      (global input) INTEGER
*          A's global column index, which points to the beginning of
*          the submatrix which is to be operated on.
*
*  DESCA   (global and local input) INTEGER array of dimension DLEN_.
*          The array descriptor for the distributed matrix A.
*
*  TAU     (local input) COMPLEX, array, dimension LOCp(M_A).
*          This array contains the scalar factors TAU of the elementary
*          reflectors computed by PCTZRZF. TAU is tied to the dis-
*          tributed matrix A.
*
*  WORK    (local workspace) COMPLEX array, dimension (LWORK)
*          LWORK = MB_A * ( Mp0 + 2*Nq0 + MB_A ), where
*          Mp0   = NUMROC( M+IROFF, MB_A, MYROW, IAROW, NPROW ) * NB_A,
*          Nq0   = NUMROC( N+ICOFF, NB_A, MYCOL, IACOL, NPCOL ) * MB_A,
*          IROFF = MOD( IA-1, MB_A ), ICOFF = MOD( JA-1, NB_A ),
*          IAROW = INDXG2P( IA, DESCA( MB_ ), MYROW, DESCA( RSRC_ ),
*                           NPROW ),
*          IACOL = INDXG2P( JA, DESCA( NB_ ), MYCOL, DESCA( CSRC_ ),
*                           NPCOL ),
*          and NUMROC, INDXG2P are ScaLAPACK tool functions.
*
*  =====================================================================
*
*     .. Parameters ..
      INTEGER            BLOCK_CYCLIC_2D, CSRC_, CTXT_, DLEN_, DT_,
     $                   LLD_, MB_, M_, NB_, N_, RSRC_
      PARAMETER          ( BLOCK_CYCLIC_2D = 1, DLEN_ = 9, DT_ = 1,
     $                     CTXT_ = 2, M_ = 3, N_ = 4, MB_ = 5, NB_ = 6,
     $                     RSRC_ = 7, CSRC_ = 8, LLD_ = 9 )
      COMPLEX            ZERO
      PARAMETER          ( ZERO = ( 0.0E+0, 0.0E+0 ) )
*     ..
*     .. Local Scalars ..
      CHARACTER          COLBTOP, ROWBTOP
      INTEGER            I, IACOL, IAROW, IB, ICOFF, ICTXT, IIA, IN,
     $                   IPT, IPV, IPW, JJA, JM1, JV, L, MYCOL, MYROW,
     $                   NPCOL, NPROW, NQ
*     ..
*     .. Local Arrays ..
      INTEGER            DESCV( DLEN_ )
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_GRIDINFO, DESCSET, INFOG2L, PCLACPY,
     $                   PCLARZB, PCLARZT, PCLASET, PTOPGET,
     $                   PTOPSET
*     ..
*     .. External Functions ..
      INTEGER            ICEIL, NUMROC
      EXTERNAL           ICEIL, NUMROC
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX, MIN, MOD
*     ..
*     .. Executable Statements ..
*
*     Get grid parameters
*
      ICTXT = DESCA( CTXT_ )
      CALL BLACS_GRIDINFO( ICTXT, NPROW, NPCOL, MYROW, MYCOL )
*
*     Quick return if possible
*
      IF( N.LT.M )
     $   RETURN
*
      L = N - M
      JM1 = JA + MIN( M+1, N ) - 1
      IN = MIN( ICEIL( IA, DESCA( MB_ ) ) * DESCA( MB_ ), IA+M-1 )
      ICOFF = MOD( JA-1, DESCA( NB_ ) )
      CALL INFOG2L( IA, JA, DESCA, NPROW, NPCOL, MYROW, MYCOL, IIA, JJA,
     $              IAROW, IACOL )
      NQ = NUMROC( N+ICOFF, DESCA( NB_ ), MYCOL, IACOL, NPCOL )
      IPV = 1
      IPT = IPV + NQ * DESCA( MB_ )
      IPW = IPT + DESCA( MB_ ) * DESCA( MB_ )
      CALL PTOPGET( ICTXT, 'Broadcast', 'Rowwise', ROWBTOP )
      CALL PTOPGET( ICTXT, 'Broadcast', 'Columnwise', COLBTOP )
      CALL PTOPSET( ICTXT, 'Broadcast', 'Rowwise', ' ' )
      CALL PTOPSET( ICTXT, 'Broadcast', 'Columnwise', 'I-ring' )
*
      CALL DESCSET( DESCV, DESCA( MB_ ), N + ICOFF, DESCA( MB_ ),
     $              DESCA( NB_ ), IAROW, IACOL, ICTXT, DESCA( MB_ ) )
*
*     Handle first block separately
*
      IB = IN - IA + 1
      JV = ICOFF + JM1 - JA + 1
*
*     Compute upper triangular matrix T
*
      CALL PCLARZT( 'Backward', 'Rowwise', L, IB, A, IA, JM1, DESCA,
     $              TAU, WORK( IPT ), WORK( IPW ) )
*
*     Copy Householder vectors into workspace
*
      CALL PCLACPY( 'All', IB, L, A, IA, JM1, DESCA, WORK( IPV ), 1,
     $              JV, DESCV )
*
*     Save temporarily strict lower part of A(IA:IA+IB-1,JA:JA+IB-1)
*
      CALL PCLACPY( 'Lower', IB-1, IB-1, A, IA+1, JA, DESCA,
     $              WORK( IPV ), 1, ICOFF+1, DESCV )
*
*     Zeroes the row panel of sub( A ) to get T(IA:IN,JA:JA+N-1)
*
      CALL PCLASET( 'All', IB, L, ZERO, ZERO, A, IA, JM1, DESCA )
      CALL PCLASET( 'Lower', IB-1, IB-1, ZERO, ZERO, A, IA+1, JA,
     $              DESCA )
*
*     Apply block Householder transformation
*
      CALL PCLARZB( 'Right', 'Conjugate transpose', 'Backward',
     $              'Rowwise', IN-IA+1, N, IB, L, WORK( IPV ), 1, JV,
     $              DESCV, WORK( IPT ), A, IA, JA, DESCA, WORK( IPW ) )
*
*     Restore strict lower part of A( IA:IA+IB-1, JA:JA+N-1 )
*
      CALL PCLACPY( 'Lower', IB-1, IB-1, WORK( IPV ), 1, ICOFF+1, DESCV,
     $              A, IA+1, JA, DESCA )
*
      DESCV( RSRC_ ) = MOD( DESCV( RSRC_ ) + 1, NPROW )
*
*     Loop over the remaining row blocks
*
      DO 10 I = IN+1, IA+M-1, DESCA( MB_ )
         IB = MIN( IA+M-I, DESCA( MB_ ) )
*
*        Compute upper triangular matrix T
*
         CALL PCLARZT( 'Backward', 'Rowwise', L, IB, A, I, JM1, DESCA,
     $                 TAU, WORK( IPT ), WORK( IPW ) )
*
*        Copy Householder vectors into workspace
*
         CALL PCLACPY( 'All', IB, L, A, I, JM1, DESCA, WORK( IPV ), 1,
     $                 JV, DESCV )
*
*        Save temporarily strict lower part of A(I:I+IB-1,J:J+IB-1 )
*
         CALL PCLACPY( 'Lower', IB-1, IB-1, A, I+1, JA+I-IA, DESCA,
     $                 WORK( IPV ), 1, ICOFF+1+I-IA, DESCV )
*
*        Zeoes the row panel of sub( A ) to get T(IA:I-1,JA+I-IA:JA+N-1)
*
         CALL PCLASET( 'All', IB, L, ZERO, ZERO, A, I, JM1, DESCA )
         CALL PCLASET( 'Lower', IB-1, IB-1, ZERO, ZERO, A, I+1, JA+I-IA,
     $                 DESCA )
*
*        Apply block Householder transformation
*
         CALL PCLARZB( 'Right', 'Conjugate transpose', 'Backward',
     $                 'Rowwise', I+IB-IA, N-I+IA, IB, L, WORK( IPV ),
     $                 1, JV, DESCV, WORK( IPT ), A, IA, JA+I-IA, DESCA,
     $                 WORK( IPW ) )
*
         CALL PCLACPY( 'Lower', IB-1, IB-1, WORK( IPV ), 1,
     $                 ICOFF+1+I-IA, DESCV, A, I+1, JA+I-IA, DESCA )
*
         DESCV( RSRC_ ) = MOD( DESCV( RSRC_ ) + 1, NPROW )
*
   10 CONTINUE
*
      CALL PTOPSET( ICTXT, 'Broadcast', 'Rowwise', ROWBTOP )
      CALL PTOPSET( ICTXT, 'Broadcast', 'Columnwise', COLBTOP )
*
      RETURN
*
*     End of PCTZRZRV
*
      END
