      SUBROUTINE PZPBLASCHK( SYMM, UPLO, N, BWL, BWU, NRHS, X, IX, JX,
     $                       DESCX, IASEED, A, IA, JA, DESCA, IBSEED,
     $                       ANORM, RESID, WORK, WORKSIZ )
*
*  -- ScaLAPACK routine (version 1.2 ALPHA) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     May 10, 1996
*
*     .. Scalar Arguments ..
      CHARACTER          SYMM, UPLO
      INTEGER            BWL, BWU, IA, IASEED, IBSEED, IX, JA, JX, N,
     $                   NRHS, WORKSIZ
      DOUBLE PRECISION   ANORM, RESID
*     ..
*     .. Array Arguments ..
      INTEGER            DESCA( * ), DESCX( * )
      COMPLEX*16         A( * ), WORK( * ), X( * )
*     ..
*
*  Purpose
*  =======
*
*  PZPBLASCHK computes the residual
*  || sub( A )*sub( X ) - B || / (|| sub( A ) ||*|| sub( X ) ||*eps*N)
*  to check the accuracy of the factorization and solve steps in the
*  LU and Cholesky decompositions, where sub( A ) denotes
*  A(IA:IA+N-1,JA,JA+N-1), sub( X ) denotes X(IX:IX+N-1, JX:JX+NRHS-1).
*
*  Notes
*  =====
*
*  A description vector is associated with each 2D block-cyclicly dis-
*  tributed matrix.  This vector stores the information required to
*  establish the mapping between a matrix entry and its corresponding
*  process and memory location.
*
*  In the following comments, the character _ should be read as
*  "of the distributed matrix".  Let A be a generic term for any 2D
*  block cyclicly distributed matrix.  Its description vector is DESCA:
*
*  NOTATION        STORED IN      EXPLANATION
*  --------------- -------------- --------------------------------------
*  DT_A   (global) DESCA( DT_ )   The descriptor type.
*  CTXT_A (global) DESCA( CTXT_ ) The BLACS context handle, indicating
*                                 the BLACS process grid A is distribu-
*                                 ted over. The context itself is glo-
*                                 bal, but the handle (the integer
*                                 value) may vary.
*  M_A    (global) DESCA( M_ )    The number of rows in the distributed
*                                 matrix A.
*  N_A    (global) DESCA( N_ )    The number of columns in the distri-
*                                 buted matrix A.
*  MB_A   (global) DESCA( MB_ )   The blocking factor used to distribute
*                                 the rows of A.
*  NB_A   (global) DESCA( NB_ )   The blocking factor used to distribute
*                                 the columns of A.
*  RSRC_A (global) DESCA( RSRC_ ) The process row over which the first
*                                 row of the matrix A is distributed.
*  CSRC_A (global) DESCA( CSRC_ ) The process column over which the
*                                 first column of A is distributed.
*  LLD_A  (local)  DESCA( LLD_ )  The leading dimension of the local
*                                 array storing the local blocks of the
*                                 distributed matrix A.
*                                 LLD_A >= MAX(1,LOCp(M_A)).
*
*  Let K be the number of rows or columns of a distributed matrix,
*  and assume that its process grid has dimension p x q.
*  LOCp( K ) denotes the number of elements of K that a process
*  would receive if K were distributed over the p processes of its
*  process column.
*  Similarly, LOCq( K ) denotes the number of elements of K that a
*  process would receive if K were distributed over the q processes of
*  its process row.
*  The values of LOCp() and LOCq() may be determined via a call to the
*  ScaLAPACK tool function, NUMROC:
*          LOCp( M ) = NUMROC( M, MB_A, MYROW, RSRC_A, NPROW ),
*          LOCq( N ) = NUMROC( N, NB_A, MYCOL, CSRC_A, NPCOL ).
*
*  Arguments
*  =========
*
*  SYMM      (global input) CHARACTER
*          if SYMM = 'H', sub( A ) is a hermitian distributed band
*          matrix, otherwise sub( A ) is a general distributed matrix.
*
*  UPLO    (global input) CHARACTER
*          if SYMM = 'H', then
*            if UPLO = 'L', the lower half of the matrix is stored
*            if UPLO = 'U', the upper half of the matrix is stored
*          if SYMM != 'S', then
*            if UPLO = 'D', the matrix is stable during factorization
*                           without interchanges
*            if UPLO != 'D', the matrix is general
*
*  N       (global input) INTEGER
*          The number of columns to be operated on, i.e. the number of
*          columns of the distributed submatrix sub( A ). N >= 0.
*
*  NRHS    (global input) INTEGER
*          The number of right-hand-sides, i.e the number of columns
*          of the distributed matrix sub( X ). NRHS >= 1.
*
*  X       (local input) COMPLEX*16 pointer into the local memory
*          to an array of dimension (LLD_X,LOCq(JX+NRHS-1). This array
*          contains the local pieces of the answer vector(s) sub( X ) of
*          sub( A ) sub( X ) - B, split up over a column of processes.
*
*  IX      (global input) INTEGER
*          X's global row index, which points to the beginning of the
*          submatrix which is to be operated on.
*
*  DESCX   (global and local input) INTEGER array of dimension DLEN_.
*          The array descriptor for the distributed matrix X.
*
*  IASEED  (global input) INTEGER
*          The seed number to generate the original matrix Ao.
*
*  JA      (global input) INTEGER
*          A's global column index, which points to the beginning of
*          the submatrix which is to be operated on.
*
*  DESCA   (global and local input) INTEGER array of dimension DLEN_.
*          The array descriptor for the distributed matrix A.
*
*  IBSEED  (global input) INTEGER
*          The seed number to generate the original matrix B.
*
*  ANORM   (global input) DOUBLE PRECISION
*          The 1-norm or infinity norm of the distributed matrix
*          sub( A ).
*
*  RESID   (global output) DOUBLE PRECISION
*          The residual error:
*          ||sub( A )*sub( X )-B|| / (||sub( A )||*||sub( X )||*eps*N).
*
*  WORK    (local workspace) COMPLEX*16 array, dimension (LWORK)
*          IF SYMM='S'
*          LWORK >= (bw+2)*bw + 2*NB
*          IF SYMM!='S'
*          LWORK >= (max(bwl, bwu) + 1)*max(bwl, bwu) + 2*NB
*
*  =====================================================================
*
*  Implemented by: Andrew J. Cleary, May 10, 1996
*
*  =====================================================================
*
*     .. Parameters ..
      COMPLEX*16         ONE
      PARAMETER          ( ONE = ( 1.0D+0, 0.0D+0 ) )
      INTEGER            BLOCK_CYCLIC_2D, CSRC_, CTXT_, DLEN_, DT_,
     $                   LLD_, MB_, M_, NB_, N_, RSRC_
      PARAMETER          ( BLOCK_CYCLIC_2D = 1, DLEN_ = 9, DT_ = 1,
     $                     CTXT_ = 2, M_ = 3, N_ = 4, MB_ = 5, NB_ = 6,
     $                     RSRC_ = 7, CSRC_ = 8, LLD_ = 9 )
*     ..
*     .. Local Scalars ..
      INTEGER            BW, IACOL, IAROW, ICTXT, IIA, IIX, INFO, IPB,
     $                   IPPRODUCT, IPW, IXCOL, IXROW, J, JJA, JJX, LDA,
     $                   MYCOL, MYROW, NP, NPCOL, NPROW, NQ, NRHS_TEMP,
     $                   START
      DOUBLE PRECISION   DIVISOR, EPS, NORMX, RESID1
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_GRIDINFO, INFOG2L, PDZNRM2, PXERBLA,
     $                   PZAXPY, PZBMATGEN, PZMATGEN, PZPBDCMV
*     ..
*     .. External Functions ..
      LOGICAL            LSAME
      INTEGER            NUMROC
      DOUBLE PRECISION   PDLAMCH
      EXTERNAL           LSAME, NUMROC, PDLAMCH
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          DBLE, MAX
*     ..
*     .. Executable Statements ..
*
*     Get needed initial parameters
*
      IF( LSAME( SYMM, 'H' ) ) THEN
         BW = BWL
         START = 1
      ELSE
         BW = MAX( BWL, BWU )
         IF( LSAME( UPLO, 'D' ) ) THEN
            START = 1
         ELSE
            START = 2
         END IF
      END IF
*
      ICTXT = DESCA( CTXT_ )
      CALL BLACS_GRIDINFO( ICTXT, NPROW, NPCOL, MYROW, MYCOL )
*
      IF( WORKSIZ.LT.( BW+2 )*BW+2*DESCA( NB_ ) ) THEN
         CALL PXERBLA( ICTXT, 'PZBLASCHK', -18 )
         RETURN
      END IF
*
      EPS = PDLAMCH( ICTXT, 'eps' )
      RESID = 0.0D+0
      DIVISOR = ANORM*EPS*DBLE( N )
*
      CALL INFOG2L( IA, JA, DESCA, NPROW, NPCOL, MYROW, MYCOL, IIA, JJA,
     $              IAROW, IACOL )
      CALL INFOG2L( IX, JX, DESCX, NPROW, NPCOL, MYROW, MYCOL, IIX, JJX,
     $              IXROW, IXCOL )
      NP = NUMROC( ( BW+1 ), DESCA( MB_ ), MYROW, 0, NPROW )
      NQ = NUMROC( N, DESCA( NB_ ), MYCOL, 0, NPCOL )
*
      IPB = 1
      IPPRODUCT = 1 + DESCA( NB_ )
      IPW = 1 + 2*DESCA( NB_ )
*
      LDA = DESCA( LLD_ )
*
*     Regenerate A
*
      IF( LSAME( SYMM, 'H' ) ) THEN
         CALL PZBMATGEN( ICTXT, UPLO, 'D', BW, BW, N, BW+1,
     $                   DESCA( NB_ ), A, DESCA( LLD_ ), 0, 0, IASEED,
     $                   MYROW, MYCOL, NPROW, NPCOL )
      ELSE
*
         CALL PZBMATGEN( ICTXT, 'N', UPLO, BWL, BWU, N, DESCA( MB_ ),
     $                   DESCA( NB_ ), A, DESCA( LLD_ ), 0, 0, IASEED,
     $                   MYROW, MYCOL, NPROW, NPCOL )
      END IF
*
*     Loop over the rhs
*
      RESID = 0.0
*
      DO 10 J = 1, NRHS
*
*           Multiply A * current column of X
*
         NRHS_TEMP = NRHS
         NRHS = 1
*
         CALL PZPBDCMV( BW+1, BW, UPLO, N, A, 1, DESCA, NRHS,
     $                  X( 1+( J-1 )*DESCX( LLD_ ) ), 1, DESCX,
     $                  WORK( IPPRODUCT ), WORK( IPW ), ( BW+2 )*BW,
     $                  INFO )
*
         NRHS = NRHS_TEMP
*
*           Regenerate column of B
*
         CALL PZMATGEN( DESCX( CTXT_ ), 'No', 'No', DESCX( M_ ),
     $                  DESCX( N_ ), DESCX( MB_ ), DESCX( NB_ ),
     $                  WORK( IPB ), DESCX( LLD_ ), DESCX( RSRC_ ),
     $                  DESCX( CSRC_ ), IBSEED, 0, NQ, J-1, 1, MYCOL,
     $                  MYROW, NPCOL, NPROW )
*
*           Figure || A * X - B || & || X ||
*
         CALL PZAXPY( N, -ONE, WORK( IPPRODUCT ), 1, 1, DESCX, 1,
     $                WORK( IPB ), 1, 1, DESCX, 1 )
*
         CALL PDZNRM2( N, NORMX, X, 1, J, DESCX, 1 )
*
         CALL PDZNRM2( N, RESID1, WORK( IPB ), 1, 1, DESCX, 1 )
*
*
*           Calculate residual = ||Ax-b|| / (||x||*||A||*eps*N)
*
         RESID1 = RESID1 / ( NORMX*DIVISOR )
*
         RESID = MAX( RESID, RESID1 )
*
   10 CONTINUE
*
      RETURN
*
*     End of PZPBLASCHK
*
      END
